
//////////////////////////////////////////////////////
//                 GLOBAL VARIABLES                 //
//////////////////////////////////////////////////////

var dimension = 2;

var X = new Array(new Array(0, 10), new Array(0, 10));

var epsilon = 0.0000000001;



//////////////////////////////////////////////////////
//                 LOCAL VARIABLES                 //
//////////////////////////////////////////////////////

var A = new Array(200);
var w = new Array(A.length);

var center = new Array(dimension);

var fac1 = 0;
var fac2 = 0;
var fac3 = 0;

var random_input = false; // true = create random input data



//////////////////////////////////////////////////////
//                   CALCULATIONS                   //
//////////////////////////////////////////////////////

// initialize problem //

function init_problem () {
  if (random_input) for (var k = 0; k < A.length; k++) {
    A[k] = new Array(10*Math.random(), 10*Math.random());
    w[k] = 10*Math.random()-4;
  } else {
    A = new Array(
      new Array(1.33, 8.89), new Array(1.89, 0.77), new Array(9.27, 1.49), new Array(9.46, 9.36), new Array(9.20, 8.69),
      new Array(7.43, 1.61), new Array(6.08, 1.34), new Array(5.57, 4.60), new Array(6.70, 2.77), new Array(8.99, 2.45),
      new Array(8.93, 7.00), new Array(8.60, 0.53), new Array(4.01, 0.31), new Array(3.34, 4.01), new Array(6.75, 5.57),
      new Array(7.36, 4.03), new Array(1.24, 6.69), new Array(3.13, 1.92), new Array(8.86, 8.74), new Array(4.18, 3.74),
      new Array(2.22, 4.35), new Array(0.88, 7.02), new Array(8.53, 7.04), new Array(6.49, 6.22), new Array(4.53, 7.87),
      new Array(4.46, 7.91), new Array(2.83, 9.88), new Array(3.39, 5.65), new Array(0.75, 4.98), new Array(7.55, 5.79),
      new Array(8.45, 0.69), new Array(3.33, 5.78), new Array(6.27, 3.66), new Array(7.31, 1.61), new Array(6.37, 7.02),
      new Array(7.23, 7.05), new Array(1.68, 6.45), new Array(3.54, 7.06), new Array(7.67, 4.17), new Array(2.20, 1.12),
      new Array(3.57, 1.99), new Array(7.34, 1.38), new Array(6.58, 4.49), new Array(5.00, 9.00), new Array(6.63, 5.23),
      new Array(5.89, 8.06), new Array(1.13, 5.25), new Array(1.90, 8.35), new Array(1.74, 1.37), new Array(9.39, 6.44)
    );
    for (var k = 0; k < A.length; k++) w[k] = (k % 10) - 4;
  }
}

// objective function //

function distance (a1, a2, b1, b2) {
  return Math.sqrt( (a1-b1)*(a1-b1) + (a2-b2)*(a2-b2) );
}

function f (x) {
  var sum = 0;
  for (var k = 0; k < A.length; k++) sum += w[k] * distance(A[k][0],A[k][1],x[0],x[1]);
  return sum;
}

// lower bounds //

function minv (v) {
  var h = v[0];
  for (var i = 1; i < v.length; i++) if (v[i] < h) h = v[i];
  return h;
}

function x_in_Y (x, y, Ix, Iy) {
  return ((Ix[0] <= x) && (x <= Ix[1]) && (Iy[0] <= y) && (y <= Iy[1]));
}

function calculate_hyperplane_fac (k,I) {
  if (x_in_Y(A[k][0],A[k][1],I[0],I[1])) {
    fac1 = 0;
    fac2 = 0;
    fac3 = 0;
  } else {
    fac1 = distance(A[k][0],A[k][1],center[0],center[1]);
    fac2 = (center[0]-A[k][0]) / fac1;
    fac3 = (center[1]-A[k][1]) / fac1;
  };
}

function hyperplane (x,y) {
  return fac1 + fac2 * (x-center[0]) + fac3 * (y-center[1]);
}

function LB (I) {
  for (var i = 0; i < dimension; i++) center[i] = 0.5 * (I[i][0]+I[i][1]);
  var v = new Array(4);
  for (var i = 0; i < v.length; i++) v[i] = 0;

  var xv = new Array(dimension);

  for (var k = 0; k < A.length; k++) {
    if (w[k] > 0) calculate_hyperplane_fac(k,I);
    for (var i = 0; i < v.length; i++) {
      for (var j = 0; j < dimension; j++) xv[j] = ( (i >> j) & 1 == 1 ? I[j][0] : I[j][1]);
      v[i] += w[k] * (w[k] > 0 ? hyperplane(xv[0],xv[1]) : distance(A[k][0],A[k][1],xv[0],xv[1]) );
    }
  }

  return minv(v);
}



//////////////////////////////////////////////////////
//              PROBLEM VISUALIZATION               //
//////////////////////////////////////////////////////

function plot_problem (y) {
  PV.set_size(0, 10, 0, 10, 1);

  PV.disc(y[0], y[1], 0.08, '#000000');
  for (var k = 0; k < A.length; k++) PV.disc(A[k][0], A[k][1], 0.05, (w[k] < 0 ? '#DE102D' : '#0059C5'));
  
  PV.text(1.3, 9.6, A.length + " demand points");
  PV.head("Problem Visualization");
}
